// (c) Copyright Microsoft Corporation.
// This source is subject to the Microsoft Permissive License.
// See http://www.microsoft.com/resources/sharedsource/licensingbasics/sharedsourcelicenses.mspx.
// All other rights reserved.


// README
//
// There are three steps to adding a property:
//
// 1. Create a member variable to store your property
// 2. Add the property to the type descriptor for this class
// 3. Add the get_ and set_ accessors for your property.
//
// Remember that all three are case sensitive!
//

Type.registerNamespace('NewPassword');

NewPassword.NewPasswordBehavior = function() {
    NewPassword.NewPasswordBehavior.initializeBase(this);
    
    // TODO : (Step 1) Add your property variables here    
    //
    var _TargetLabelId;
    
    this.initialize = function() {
        NewPassword.NewPasswordBehavior.callBaseMethod(this, 'initialize');
       
        this.control.element.attachEvent('onkeyup', Function.createDelegate(this, this._onkeyup));
        
    }
    
    this.dispose = function() {
       
        // TODO: add your cleanup code here
        
        NewPassword.NewPasswordBehavior.callBaseMethod(this, 'dispose');
    }
    
    this.getDescriptor = function() {
        var td = NewPassword.NewPasswordBehavior.callBaseMethod(this, 'getDescriptor');
        td.addProperty('TargetLabelId', String);        
        return td;
    }
    
        this.get_TargetLabelId = function() {
            return _TargetLabelId;
        }
        
        this.set_TargetLabelId = function(value) {
            _TargetLabelId = value;
        }
        
        
    // These are helper functions for communicating state back to the extender on the
    // server side.  They take or return a custom string that is available in your initialize method
    // and later.
    //
    this.getClientState = function() {
        var value = NewPassword.NewPasswordBehavior.callBaseMethod(this, 'get_ClientState');                
        if (value == '') value = null;
        return value;
    }
     
    this.setClientState = function(value) {
        return NewPassword.NewPasswordBehavior.callBaseMethod(this, 'set_ClientState',[value]);                
    }
    
     // Custom Behavior
    this._onkeyup = function() { 
        var lbl = document.getElementById(_TargetLabelId);
        var passwd = this.control.element.value;
        var intScore   = 0
		var strVerdict = "No Password"
        
		// PASSWORD LENGTH
		if (passwd.length<5)                         // length 4 or less
		{
			intScore = (intScore+3)
		}
		else if (passwd.length>4 && passwd.length<8) // length between 5 and 7
		{
			intScore = (intScore+6)
		}
		else if (passwd.length>7 && passwd.length<16)// length between 8 and 15
		{
			intScore = (intScore+12)
		}
		else if (passwd.length>15)                    // length 16 or more
		{
			intScore = (intScore+18)
		}
		
		
		// LETTERS (Not exactly implemented as dictacted above because of my limited understanding of Regex)
		if (passwd.match(/[a-z]/))                              // [verified] at least one lower case letter
		{
			intScore = (intScore+1)
		}
		
		if (passwd.match(/[A-Z]/))                              // [verified] at least one upper case letter
		{
			intScore = (intScore+5)
		}
		
		// NUMBERS
		if (passwd.match(/\d+/))                                 // [verified] at least one number
		{
			intScore = (intScore+5)
		}
		
		if (passwd.match(/(.*[0-9].*[0-9].*[0-9])/))             // [verified] at least three numbers
		{
			intScore = (intScore+5)
		}
		
		
		// SPECIAL CHAR
		if (passwd.match(/.[!,@,#,$,%,^,&,*,?,_,~]/))            // [verified] at least one special character
		{
			intScore = (intScore+5)
		}
		
																 // [verified] at least two special characters
		if (passwd.match(/(.*[!,@,#,$,%,^,&,*,?,_,~].*[!,@,#,$,%,^,&,*,?,_,~])/))
		{
			intScore = (intScore+5)
		}
	
		
		// COMBOS
		if (passwd.match(/([a-z].*[A-Z])|([A-Z].*[a-z])/))        // [verified] both upper and lower case
		{
			intScore = (intScore+2)
		}

		if (passwd.match(/(\d.*\D)|(\D.*\d)/))                    // [FAILED] both letters and numbers, almost works because an additional character is required
		{
			intScore = (intScore+2)
		}
 
																  // [verified] letters, numbers, and special characters
		if (passwd.match(/([a-zA-Z0-9].*[!,@,#,$,%,^,&,*,?,_,~])|([!,@,#,$,%,^,&,*,?,_,~].*[a-zA-Z0-9])/))
		{
			intScore = (intScore+2)
		}
	
	
		if(intScore < 16)
		{
		   strVerdict = "Very Weak Password"
		   lbl.style.backgroundColor  = "red";
		}
		else if (intScore > 15 && intScore < 25)
		{
		   strVerdict = "Weak Password"
		   lbl.style.backgroundColor  = "tomato";
		}
		else if (intScore > 24 && intScore < 35)
		{
		   strVerdict = "Mediocre Password Strength"
		   lbl.style.backgroundColor  = "yellow";
		}
		else if (intScore > 34 && intScore < 45)
		{
		   strVerdict = "Strong Password"
		   lbl.style.backgroundColor  = "palegreen";
		}
		else
		{
		   strVerdict = "Very Strong Password"
		   lbl.style.backgroundColor  = "lawngreen";
		}
		
	
      lbl.innerHTML = strVerdict;  

    }   
}

NewPassword.NewPasswordBehavior.registerSealedClass('NewPassword.NewPasswordBehavior', Microsoft.AtlasControlExtender.BehaviorBase);
Sys.TypeDescriptor.addType('NewPassword'.toLowerCase() /* Safari Compat */, 'NewPasswordBehavior', NewPassword.NewPasswordBehavior);

	
/* ******************************************************************
Password Strength Algorithm is based on JavaScript 
that is Copyright (c) 2006 Steve Moitozo 
<god at zilla dot us>

Permission is hereby granted, free of charge, to any person 
obtaining a copy of this software and associated documentation 
files (the "Software"), to deal in the Software without 
restriction, including without limitation the rights to use, 
copy, modify, merge, publish, distribute, sublicense, and/or 
sell copies of the Software, and to permit persons to whom the 
Software is furnished to do so, subject to the following 
conditions:

   The above copyright notice and this permission notice shall 
be included in all copies or substantial portions of the 
Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY 
KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE 
WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE 
OR OTHER DEALINGS IN THE SOFTWARE. 
---------------------------------------------------------------


Password Strength Factors and Weightings

password length:
level 0 (3 point): less than 4 characters
level 1 (6 points): between 5 and 7 characters
level 2 (12 points): between 8 and 15 characters
level 3 (18 points): 16 or more characters

letters:
level 0 (0 points): no letters
level 1 (5 points): all letters are lower case
level 2 (7 points): letters are mixed case

numbers:
level 0 (0 points): no numbers exist
level 1 (5 points): one number exists
level 1 (7 points): 3 or more numbers exists

special characters:
level 0 (0 points): no special characters
level 1 (5 points): one special character exists
level 2 (10 points): more than one special character exists

combinatons:
level 0 (1 points): letters and numbers exist
level 1 (1 points): mixed case letters
level 1 (2 points): letters, numbers and special characters 
					exist
level 1 (2 points): mixed case letters, numbers and special 
					characters exist

************************************************************ */